@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/consumers/ascii_writer/tsp_ascii_writer.h,v 1.4 2004/11/09 22:33:01 erk Exp $
+$Header: /cvsroot/tsp/tsp/src/consumers/ascii_writer/tsp_ascii_writer.h,v 1.7 2005/11/01 17:07:48 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -37,10 +37,14 @@ Purpose   : TSP ascii writer consumer
 #ifndef _TSP_ASCII_WRITER_H_
 #define _TSP_ASCII_WRITER_H_
 
-#include "tsp_abs_types.h"
-#include "tsp_consumer.h"
 #include <pthread.h>
-#include <stdio.h>
+#include <stdio.h> 
+
+#include <tsp_abs_types.h>
+#include <tsp_consumer.h>
+
+
+typedef int32_t  (* tsp_ascii_writer_logMsg_ft)(char* fmt, ...);
 
 /**
  * @defgroup Ascii_Writer
@@ -56,6 +60,10 @@ extern int tsp_ascii_writer_colno;
 extern int tsp_ascii_writer_nb_var;
 extern int tsp_ascii_writer_current_var;
 
+void tsp_ascii_writer_set_logMsgCB(tsp_ascii_writer_logMsg_ft logMsgCB);
+
+int32_t tsp_ascii_writer_logMsg_stdout(char* fmt, ...);
+
 /**
  * Initialise ascii TSP consumer.
  * We must pass main arguments to TSP lib for
@@ -96,6 +104,25 @@ int32_t
 tsp_ascii_writer_load_config(const char* conffilename, 
 			     TSP_consumer_symbol_requested_t**  tsp_symbols,
 			     int32_t* nb_symbols);
+
+/**
+ * Make the requested symbol unique in the provided list.
+ * The unicity may be enforced by reducing the number 
+ * of requested symbols if ever duplicate symbols have
+ * same name, period and phase.
+ * @param tsp_symbols INOUT, pointer to the array of symbols 
+ *                           to be checked for unicity on entry,
+ *                           the array of unique symbols on return.
+ * @param nb_symbols INOUT, the number of symbols in the provided array on entry
+ *                          the number of remaining (unique) symbols on return.
+ * @return 0 if uniticy may be enforced or the rank of non-unique
+ *           symbol otherwise.
+ * @ingroup Ascii_Writer
+ */
+int32_t 
+tsp_ascii_writer_make_unique(TSP_consumer_symbol_requested_t**  tsp_symbols,
+			     int32_t* nb_symbols);
+
 /**
  * Validate the requested symbol against the TSP provider 
  * located on tsp_provider_hostname.
