@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Id: gdisp_kernel.h,v 1.14 2005/10/05 19:21:00 esteban Exp $
+$Id: gdisp_kernel.h,v 1.18 2006/01/20 21:59:14 esteban Exp $
 
 -----------------------------------------------------------------------
 
@@ -53,6 +53,7 @@ File      : Graphic Tool Kernel Interface.
  */
 #include "tsp_const_def.h"
 #include "tsp_consumer.h"
+#include "tsp_hash.h"
 #include "gdisp_doubleArray.h"
 
 
@@ -61,6 +62,11 @@ File      : Graphic Tool Kernel Interface.
  */
 #undef _DND_DEBUG_
 
+/*
+ * Few defines.
+ */
+#undef GD_LOAD_CONFIGURATION_WITH_ALL_SYMBOLS
+#define GD_UNREF_THINGS
 
 /*
  * 10 out of 170 colors.
@@ -170,7 +176,7 @@ typedef enum {
 typedef struct Symbol_T_ {
 
   guchar                           sReference;
-  guint                            sConfIndex;
+  guint                            sPgi;
   TSP_consumer_symbol_requested_t  sInfo;
   guint                            sTimeTag;
   gdouble                          sLastValue;
@@ -262,6 +268,8 @@ typedef struct Provider_T_ {
 
   gint              pSymbolNumber;
   Symbol_T         *pSymbolList;
+  hash_t           *pSymbolHashTable;    /* name -> pointer */
+  hash_t           *pSymbolHashTablePGI; /* pgi  -> pointer */
 
   SampleList_T      pSampleList;
   guint             pLoad;
@@ -276,8 +284,10 @@ typedef struct Provider_T_ {
   /*
    * Configuration temporarily data.
    */
+#if defined(GD_LOAD_CONFIGURATION_WITH_ALL_SYMBOLS)
   void             *pSymbolInConfiguration;
   guint             pNbSymbolInConfiguration;
+#endif
 
   /*
    * Graphic information.
@@ -465,6 +475,8 @@ typedef struct Page_T_ {
 
 typedef struct KernelWidget_T_ {
 
+  /* -------------------- MAIN BOARD WIDGETS -------------------- */
+
   GtkWidget         *mainBoardWindow;
   gint               mainBoardWindowXPosition;
   gint               mainBoardWindowYPosition;
@@ -472,19 +484,38 @@ typedef struct KernelWidget_T_ {
   GtkWidget         *mainBoardStopButton;
   GtkWidget         *mainBoardOutputList;
   guint              mainBoardOutputListSize;
+  GtkWidget         *mainBoardLogoArea;
+  GdkGC             *mainBoardLogoContext;
+#define GD_ANIMATED_LOGO_NB 11
+  Pixmap_T          *mainBoardLogoTable[GD_ANIMATED_LOGO_NB];
+  Pixmap_T         **mainBoardCurrentLogo;
+  Pixmap_T          *mainBoardLargeLogo;
+  gint               mainBoardLargeLogoX;
+  gboolean           mainBoardLargeLogoGoesLeft;
+  GtkWidget         *mainBoardMenuBar;
+
+  /* -------------------- PILOT BOARD WIDGETS -------------------- */
+
   Pixmap_T          *pilotBoardDigitPixmap;
   GtkWidget         *pilotBoardTimeArea;
   GdkGC             *pilotBoardTimeContext;
   GtkWidget         *pilotBoardElapsedTimeArea;
   GdkGC             *pilotBoardElapsedTimeContext;
+
+  /* -------------------- DATA BOOK WIDGETS ---------------------- */
+
   GtkWidget         *dataBookWindow;
   gint               dataBookWindowXPosition;
   gint               dataBookWindowYPosition;
   GtkWidget         *dataBookWidget;
   GtkWidget         *dataBookApplyButton;
+
+  /* ----------------- WIDGETS IN DATA BOOK PAGES ----------------- */
+
 #define _SYMBOL_CLIST_COLUMNS_NB_ 3
   GtkWidget         *symbolCList;
   GtkWidget         *symbolFrame;
+  GtkWidget         *providerVBox;
   GtkWidget         *pRadioButton;
   GtkWidget         *naRadioButton;
   GtkWidget         *ndRadioButton;
@@ -497,6 +528,10 @@ typedef struct KernelWidget_T_ {
   Pixmap_T          *expandedNodePixmap;
   Pixmap_T          *collapsedNodePixmap;
   GtkWidget         *sampledSymbolHTree;
+  GtkWidget         *sampledSymbolScrolledWindow;
+  GtkWidget         *fileSelector;
+
+  /* ------------------------ PIXMAP TABLE ------------------------ */
 
   GList             *pixmapTable;
 
@@ -517,6 +552,8 @@ typedef struct Kernel_T_ {
   /*
    * GTK timer identity and period in milli-seconds.
    */
+  gint               logoTimerIdentity;
+  gint               logoHighSpeedTimerIdentity;
   gint               stepTimerIdentity;
 #define GD_TIMER_MIN_PERIOD 100
   guint              stepTimerPeriod;
@@ -554,6 +591,8 @@ typedef struct Kernel_T_ {
 					       void(*action)(Kernel_T_Ptr));
   void             (*assignSymbolsToProviders)(Kernel_T_Ptr);
   PlotType_T       (*getPlotTypeFromPlotName) (Kernel_T_Ptr,gchar*);
+  void             (*startSamplingProcess)    (Kernel_T_Ptr);
+  void             (*stopSamplingProcess)     (Kernel_T_Ptr);
 
   /*
    * Host/URL management.
@@ -566,7 +605,15 @@ typedef struct Kernel_T_ {
    */
   GList             *providerList;
   SortingMethod_T    sortingMethod;
+  guint              providerId;
 
+  /*
+   * IO management.
+   */
+  gchar             *ioFilename;
+  gboolean         (*factoryAction)(Kernel_T_Ptr);
+  guint              factoryActionId;
+  gboolean           retreiveAllSymbols;
 
   /*
    * -------------------- Graphic part --------------------
