@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Id: gdisp_pages.c,v 1.7 2005/10/05 19:21:00 esteban Exp $
+$Id: gdisp_pages.c,v 1.11 2006/02/02 21:03:32 esteban Exp $
 
 -----------------------------------------------------------------------
 
@@ -1517,9 +1517,12 @@ gdisp_finalizeGraphicPageCreation (Kernel_T *kernel,
   /*
    * Set up window size, title and border width.
    */
+#define GD_WINDOW_MAX_INITIAL_WIDTH  400
+#define GD_WINDOW_MAX_INITIAL_HEIGHT 300
+
   gtk_widget_set_usize(GTK_WIDGET(newPage->pWindow),
-		       3 * screenWidth  / 4,  /* width  */
-		       3 * screenHeight / 4); /* height */
+		       MIN(3 * screenWidth  / 4,GD_WINDOW_MAX_INITIAL_WIDTH),
+		       MIN(3 * screenHeight / 4,GD_WINDOW_MAX_INITIAL_HEIGHT));
 
   gtk_window_set_title(GTK_WINDOW(newPage->pWindow),
 		       newPage->pName->str);
@@ -1532,9 +1535,12 @@ gdisp_finalizeGraphicPageCreation (Kernel_T *kernel,
 
   gtk_widget_show(newPage->pWindow);
 
+#if defined(GD_KEEP_ASPECT_RATIO)
+  /* FIXME : should be a user preference */
   gdk_window_set_geometry_hints(GTK_WIDGET(newPage->pWindow)->window,
 				&pWindowHints,
 				GDK_HINT_ASPECT);
+#endif
 
   /*
    * Create a table with the correct dimensions.
@@ -1545,6 +1551,7 @@ gdisp_finalizeGraphicPageCreation (Kernel_T *kernel,
 
   gtk_table_set_row_spacings(GTK_TABLE(newPage->pTable),
 			     GD_PAGE_ROW_SPACINGS);
+
   gtk_table_set_col_spacings(GTK_TABLE(newPage->pTable),
 			     GD_PAGE_COL_SPACINGS);
 
