@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Id: gdisp_sampledSymbols.c,v 1.7 2005/10/05 19:21:01 esteban Exp $
+$Id: gdisp_sampledSymbols.c,v 1.9 2006/01/20 21:59:14 esteban Exp $
 
 -----------------------------------------------------------------------
 
@@ -375,7 +375,7 @@ gdisp_createProviderNode(Kernel_T      *kernel,
 				pPixmap->mask,
 				pPixmap->pixmap,
 				pPixmap->mask,
-				FALSE, /* is a leave  */
+  /* FIXME for the moment */    TRUE, /* is a leave  */
 				FALSE); /* is expanded */
 
   gtk_ctree_node_set_selectable(GTK_CTREE(tree),
@@ -647,17 +647,22 @@ static void
 gdisp_poolSampledSymbolList ( Kernel_T *kernel )
 {
 
-  GtkWidget        *cTree            =    (GtkWidget*)NULL;
-  GString          *messageString    =      (GString*)NULL;
-  GList            *providerItem     =        (GList*)NULL;
-  Provider_T       *provider         =   (Provider_T*)NULL;
-  Symbol_T         *symbol           =     (Symbol_T*)NULL;
-  GtkCTreeNode     *pSymbolAnchor    = (GtkCTreeNode*)NULL;
+  GtkWidget        *cTree             =    (GtkWidget*)NULL;
+  GString          *messageString     =      (GString*)NULL;
+  GList            *providerItem      =        (GList*)NULL;
+  Provider_T       *provider          =   (Provider_T*)NULL;
+  Symbol_T         *symbol            =     (Symbol_T*)NULL;
+  GtkCTreeNode     *pSymbolAnchor     = (GtkCTreeNode*)NULL;
+  GtkCTreeNode     *pNode             = (GtkCTreeNode*)NULL;
+  GtkCTreeNode     *sNode             = (GtkCTreeNode*)NULL;
 
-  gint              pSampleCpt       =                   0;
-  SampleList_T     *pSampleList      = (SampleList_T*)NULL;
-  guint             pSampleMax       =                   0;
+  gint              pSampleCpt        =                   0;
+  SampleList_T     *pSampleList       = (SampleList_T*)NULL;
+  guint             pSampleMax        =                   0;
 
+#define GD_SAMPLE_PGI_AS_STRING_LENGTH 10
+  gchar             samplePGIasStringBuffer[GD_SAMPLE_PGI_AS_STRING_LENGTH];
+  gchar            *samplePGIasString =        (gchar*)NULL;
 
   /* ------------------------ PER PROVIDER ---------------------- */
 
@@ -685,79 +690,135 @@ gdisp_poolSampledSymbolList ( Kernel_T *kernel )
     /*
      * Look for symbol anchor.
      */
-    pSymbolAnchor = gdisp_getChildAccordingToItsName(kernel,
-						     provider->pNode,
-						     "sAnchor");
+    if (provider->pNode != (GtkCTreeNode*)NULL) {
 
-    if (pSymbolAnchor == (GtkCTreeNode*)NULL) {
+      pSymbolAnchor = gdisp_getChildAccordingToItsName(kernel,
+						       provider->pNode,
+						       "sAnchor");
 
-      /* should never happen, because I did create this node !! */
-      messageString = g_string_new((gchar*)NULL);
-      g_string_sprintf(messageString,
-		       "%s provider has no anchor for symbols.",
-		       provider->pUrl->str);
-      kernel->outputFunc(kernel,messageString,GD_ERROR);
+      if (pSymbolAnchor == (GtkCTreeNode*)NULL) {
 
-    }
-    else {
+	/* should never happen, because I did create this node !! */
+	messageString = g_string_new((gchar*)NULL);
+	g_string_sprintf(messageString,
+			 "%s provider has no anchor for symbols.",
+			 provider->pUrl->str);
+	kernel->outputFunc(kernel,messageString,GD_ERROR);
 
-      /* --------------------- PER SAMPLED SYMBOLS ---------------- */
+      }
+      else {
 
-      /*
-       * Loop over all sampled symbol of the current provider.
-       */
-      pSampleList = &provider->pSampleList;
-      pSampleMax  = pSampleList->len;
-
-      for (pSampleCpt=0; pSampleCpt<pSampleMax; pSampleCpt++) {
+	/* --------------------- PER SAMPLED SYMBOLS ---------------- */
 
 	/*
-	 * Get in touch with the symbol through the global index.
+	 * Loop over all sampled symbol of the current provider.
 	 */
-	symbol = &provider->pSymbolList[pSampleList->val[pSampleCpt].index];
+	pSampleList = &provider->pSampleList;
+	pSampleMax  = pSampleList->len;
 
-	/*
-	 * If referenced... ie, used by graphic plots...
-	 */
-	if (symbol->sReference > 0) {
+	for (pSampleCpt=0; pSampleCpt<pSampleMax; pSampleCpt++) {
 
 	  /*
-	   * Create the hierarchy for that symbol, if not already done.
+	   * Get in touch with the symbol through the global index.
 	   */
-	  if (symbol->sNode == (GtkCTreeNode*)NULL) {
+	  if (pSampleList->val[pSampleCpt].index >= 0) {
 
-	    gdisp_createSymbolNode(kernel,
-				   cTree,
-				   pSymbolAnchor,
-				   symbol);
+#if defined(GD_LOAD_CONFIGURATION_WITH_ALL_SYMBOLS)
 
-	  }
-	  else {
+	    symbol =
+	      &provider->pSymbolList[pSampleList->val[pSampleCpt].index];
 
-	    gdisp_updateSymbolNode(kernel,
-				   cTree,
-				   symbol);
+#else
 
-	  }
+	    /*
+	     * Convert PGI as an unsigned integer to a string.
+	     */
+	    samplePGIasStringBuffer[GD_SAMPLE_PGI_AS_STRING_LENGTH-1] = '\0';
+	    samplePGIasString =
+	      gdisp_uIntToStr(pSampleList->val[pSampleCpt].index,
+		  &samplePGIasStringBuffer[GD_SAMPLE_PGI_AS_STRING_LENGTH-1]);
 
-	} /* sReference > 0 */
+	    /*
+	     * Retreive target symbol.
+	     */
+	    if (provider->pSymbolHashTablePGI == (hash_t*)NULL) {
 
-	else {
+	      symbol = (Symbol_T*)NULL;
 
-	  if (symbol->sNode != (GtkCTreeNode*)NULL) {
+	    }
+	    else {
 
-	    gtk_ctree_remove_node(GTK_CTREE(cTree),
-				  symbol->sNode);
+	      symbol = (Symbol_T*)
+		hash_get(provider->pSymbolHashTablePGI,samplePGIasString);
 
-	    symbol->sNode = (GtkCTreeNode*)NULL;
+	    }
 
-	  }
+#endif
 
-	} /* sReference == 0 */
+	    /*
+	     * If referenced... ie, used by graphic plots...
+	     */
+	    if (symbol != (Symbol_T*)NULL) {
 
-      } /* loop over sampled symbols */
+	      if (symbol->sReference > 0) {
 
-    } /* found sAnchor */
+		/*
+		 * Create the hierarchy for that symbol, if not already done.
+		 */
+		if (symbol->sNode == (GtkCTreeNode*)NULL) {
+
+		  gdisp_createSymbolNode(kernel,
+					 cTree,
+					 pSymbolAnchor,
+					 symbol);
+
+		}
+		else {
+
+		  gdisp_updateSymbolNode(kernel,
+					 cTree,
+					 symbol);
+
+		}
+
+	      } /* sReference > 0 */
+
+	      else {
+
+		if (symbol->sNode != (GtkCTreeNode*)NULL) {
+
+		  gtk_ctree_remove_node(GTK_CTREE(cTree),
+					symbol->sNode);
+
+		  symbol->sNode = (GtkCTreeNode*)NULL;
+
+		}
+
+	      } /* sReference == 0 */
+
+	    } /* symbol not null */
+
+	  } /* if (index >= 0) */
+
+	} /* loop over sampled symbols */
+
+      } /* found sAnchor */
+
+    } /* pNode is not null */
+
+    else {
+
+      /*
+       * Create a node that will contain all provider information.
+       */
+      gdisp_createProviderNode(kernel,
+			       kernel->widgets.sampledSymbolScrolledWindow,
+			       cTree,
+			       provider,
+			       &pNode,
+			       &sNode);
+
+    }
 
     /*
      * Next provider.
@@ -884,6 +945,8 @@ gdisp_createSampledSymbolList ( Kernel_T  *kernel,
   gtk_container_add(GTK_CONTAINER(sFrame),scrolledWindow); 
   gtk_widget_show(scrolledWindow);
 
+  kernel->widgets.sampledSymbolScrolledWindow = scrolledWindow;
+
 
   /* --------- HIERARCHICAL TREE FOR HANDLING ALL PROVIDERS --------- */
 
@@ -962,6 +1025,9 @@ gdisp_createSampledSymbolList ( Kernel_T  *kernel,
 
     kernel->widgets.selectedNodeStyle =
       gtk_style_copy(gtk_widget_get_default_style());
+    
+    kernel->widgets.selectedNodeStyle =
+      gtk_style_ref(kernel->widgets.selectedNodeStyle);
 
     kernel->widgets.selectedNodeStyle->font = selectedFont;
 #if defined(GD_TREE_WITH_COLOR)
@@ -972,6 +1038,9 @@ gdisp_createSampledSymbolList ( Kernel_T  *kernel,
     kernel->widgets.unselectedNodeStyle =
       gtk_style_copy(gtk_widget_get_default_style());
 
+    kernel->widgets.unselectedNodeStyle =
+      gtk_style_ref(kernel->widgets.unselectedNodeStyle);
+
     kernel->widgets.unselectedNodeStyle->font = unselectedFont;
 #if defined(GD_TREE_WITH_COLOR)
     kernel->widgets.unselectedNodeStyle->fg[GTK_STATE_NORMAL] =
@@ -989,6 +1058,89 @@ gdisp_createSampledSymbolList ( Kernel_T  *kernel,
 }
 
 
+/*
+ * Refresh provider graphic list.
+ */
+void
+gdisp_refreshSampledSymbolList ( Kernel_T *kernel )
+{
+
+  GtkWidget    *cTree        = (GtkWidget*)NULL;
+  GList        *cTreeList    = (GList*)NULL;
+  GtkCTreeRow  *cTreeRow     = (GtkCTreeRow*)NULL;
+  GtkCTreeNode *cTreeNode    = (GtkCTreeNode*)NULL;
+  guint         cTreeLength  = 0;
+  guint         cTreeCpt     = 0;
+
+  /* ------------------------ PER PROVIDER ---------------------- */
+
+  cTree = kernel->widgets.sampledSymbolHTree;
+
+  /*
+   * When removing a node, GTK main loop activate the "unselect"
+   * callback, that performs a post recursive action on a node
+   * that has been destroyed.
+   * Avoid that shame by temporarily blocking the signal emission.
+   */
+  gtk_clist_freeze(GTK_CLIST(cTree));
+  gtk_signal_handler_block_by_func(GTK_OBJECT(cTree),
+				   gdisp_treeUnselectRowCallback,
+				   (gpointer)kernel); 
+
+  /*
+   * Remove all top-level node, those that have no parent.
+   * First step : build top-level list.
+   */
+  cTreeLength = g_list_length(GTK_CTREE(cTree)->clist.row_list);
+
+  for (cTreeCpt=0; cTreeCpt<cTreeLength; cTreeCpt++) {
+
+    cTreeNode = gtk_ctree_node_nth(GTK_CTREE(cTree),cTreeCpt);
+
+    if (cTreeNode != (GtkCTreeNode*)NULL) {
+
+      cTreeRow = GTK_CTREE_ROW(cTreeNode);
+
+      if (cTreeRow->parent == (GtkCTreeNode*)NULL) {
+
+	cTreeList = g_list_append(cTreeList,(gpointer)cTreeNode);
+
+      } /* node is top-level, ie without parent */
+
+    } /* node exists */
+
+  } /* loop over all rows */
+
+  /*
+   * Second step : remove all top_level nodes.
+   */
+  cTreeList = g_list_first(cTreeList);
+  while (cTreeList != (GList*)NULL) {
+
+    gtk_ctree_remove_node(GTK_CTREE(cTree),
+			  (GtkCTreeNode*)cTreeList->data);
+
+    cTreeList = g_list_next(cTreeList);
+
+  }
+
+  /*
+   * Finalise.
+   */
+  gdisp_finaliseHierarchicalTree(kernel,
+				 cTree);
+
+  /*
+   * Activate again the unselect handler.
+   */
+  gtk_signal_handler_unblock_by_func(GTK_OBJECT(cTree),
+				     gdisp_treeUnselectRowCallback,
+				     (gpointer)kernel);
+  gtk_clist_thaw(GTK_CLIST(cTree));
+
+}
+
+
 /*
  * Destroy GDISP+ sampled symbol list.
  */
@@ -1004,10 +1156,14 @@ gdisp_destroySampledSymbolList ( Kernel_T *kernel )
   /*
    * Destroy tree specific styles.
    */
+#if defined(GD_UNREF_THINGS)
   gtk_style_unref(kernel->widgets.selectedNodeStyle);
+#endif
   kernel->widgets.selectedNodeStyle = (GtkStyle*)NULL;
 
+#if defined(GD_UNREF_THINGS)
   gtk_style_unref(kernel->widgets.unselectedNodeStyle);
+#endif
   kernel->widgets.unselectedNodeStyle = (GtkStyle*)NULL;
 
   /*
