@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Id: tsp_provider.c,v 1.31 2005/11/01 10:37:56 erk Exp $
+$Id: tsp_provider.c,v 1.35 2006/01/22 09:35:15 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -42,6 +42,7 @@ Purpose   : Main implementation for the producer module
 #include <tsp_glu.h>	
 #include <tsp_time.h>
 #include <tsp_common.h>
+#include "tsp_datapool.h"
 
 /** modified argc and argv, will be returned to user code after init */
 static  char** X_argv = 0;
@@ -223,8 +224,6 @@ static int TSP_cmd_line_parser(int* argc, char** argv[])
   STRACE_IO(("-->OUT"));
   
   return ret;
-
-
 }
 
 int TSP_provider_is_initialized(void)
@@ -424,7 +423,9 @@ void  TSP_provider_request_sample(TSP_request_sample_t* req_sample,
 {
   TSP_LOCK_MUTEX(&X_tsp_request_mutex,);	
   STRACE_IO(("-->IN"));
-
+  int32_t i;
+  int32_t invalid_period;
+  int32_t invalid_phase;
   ans_sample->version_id            = TSP_VERSION;
   ans_sample->channel_id            = req_sample->channel_id;
   ans_sample->status                = TSP_STATUS_ERROR_UNKNOWN;
@@ -440,18 +441,41 @@ void  TSP_provider_request_sample(TSP_request_sample_t* req_sample,
   
   if(req_sample->version_id <= TSP_VERSION)
     {
+      
       if(TSP_session_get_symbols_global_index_by_channel(req_sample->channel_id, &(req_sample->symbols) ))
-	{     
-	  /* The datapool will be created here (if it does not already exist) */
-	  if(TSP_session_create_symbols_table_by_channel(req_sample, ans_sample)) 
-	    {
-	      ans_sample->status = TSP_STATUS_OK;
-	      firstGLU->start(firstGLU); /* FIXME Y.D : Why start now the thread */
+	{  
+	  invalid_period = 0;
+	  invalid_phase  = 0;
+	  /* validate period and phase range */
+	  for (i=0;i<req_sample->symbols.TSP_sample_symbol_info_list_t_len;++i) {	    
+	    if(req_sample->symbols.TSP_sample_symbol_info_list_t_val[i].period < 1) {
+	      req_sample->symbols.TSP_sample_symbol_info_list_t_val[i].provider_global_index = -1;
+	      invalid_period++;
 	    }
-	  else  
-	    {
-	      STRACE_ERROR(("Function TSP_session_create_symbols_table_by_channel failed"));
-	    }        
+	    if(req_sample->symbols.TSP_sample_symbol_info_list_t_val[i].phase < 0) {
+	      req_sample->symbols.TSP_sample_symbol_info_list_t_val[i].provider_global_index = -1;
+	      invalid_phase++;
+	    }
+	  }
+	  if ((invalid_phase>0) || (invalid_period>0)) {
+	    ans_sample->status = TSP_STATUS_ERROR_UNKNOWN;
+	    TSP_common_sample_symbol_info_list_copy(&(ans_sample->symbols), req_sample->symbols);
+	    STRACE_DEBUG(("Invalid phase or period"));
+	  } else {
+
+	    /* The datapool will be created here (if it does not already exist) */
+	    if(TSP_session_create_symbols_table_by_channel(req_sample, ans_sample)) 
+	      {
+		/* FIXME do we need to start glu here for PASSIVE glu 
+		 * instead of TSP_provider_private_run
+		 */
+		ans_sample->status = TSP_STATUS_OK;
+	      }
+	    else  
+	      {
+		STRACE_ERROR(("Function TSP_session_create_symbols_table_by_channel failed"));
+	      }    
+	  }    
 	}
       else
 	{
@@ -575,7 +599,6 @@ void* TSP_provider_garbage_collector_thread(void* dummy)
    return (void*)NULL;
 }
 
-
 int TSP_provider_private_init(GLU_handle_t* theGLU, int* argc, char** argv[])
 {
   int ret = TRUE;
@@ -612,6 +635,20 @@ int TSP_provider_private_init(GLU_handle_t* theGLU, int* argc, char** argv[])
   return ret;
 } /* End of TSP_provider_private_init */
 
+int TSP_provider_private_run() {
+  /* instantiate datapool */
+  TSP_global_datapool_get_instance(firstGLU);
+  /* Launch GLU now 
+   */
+  /* FIXME do we need to differentiate 
+   * PASSIVE and ACTIVE GLU ?
+   */
+  /*  if (GLU_SERVER_TYPE_ACTIVE == firstGLU->type) { */
+    firstGLU->start(firstGLU);
+    /* } */
+  return 0;
+}
+
 
 int TSP_provider_request_async_sample_write(TSP_async_sample_t* async_sample_write)
 {
@@ -654,7 +691,7 @@ int TSP_provider_request_async_sample_read(TSP_async_sample_t* async_sample_read
     ret = TRUE;
   }
 
-  STRACE_DEBUG(("TSP_PROVIDER After async_read: pgi %d value %f return %d ",async_sample_read->provider_global_index,(double*)(async_sample_read->data.data_val),ret ));
+  STRACE_DEBUG(("TSP_PROVIDER After async_read: pgi %d value %f return %d ",async_sample_read->provider_global_index,*((double*)(async_sample_read->data.data_val)),ret ));
   
   STRACE_IO(("-->OUT"));
   return ret;
