@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/core/driver/tsp_consumer.c,v 1.37 2005/10/30 17:18:18 erk Exp $
+$Header: /cvsroot/tsp/tsp/src/core/driver/tsp_consumer.c,v 1.41 2006/01/27 17:23:59 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -398,8 +398,8 @@ void TSP_consumer_end(void)
   STRACE_IO(("-->OUT"));
 }
 
-TSP_provider_t* TSP_consumer_connect_url(const char*  url)
-{	
+TSP_provider_t* 
+TSP_consumer_connect_url(const char*  url) {	
   TSP_provider_t *provider = NULL;
   TSP_server_t server;
   TSP_server_info_string_t server_info;
@@ -407,6 +407,7 @@ TSP_provider_t* TSP_consumer_connect_url(const char*  url)
   int i, servernumber;
   char url_tok[2*MAXHOSTNAMELEN], url_lkup[2*MAXHOSTNAMELEN];
   char *protocol, *hostname, *servername, *p;
+  char *endptr;
 
   /** Parse (simply ...) URL **/ 
   protocol = NULL;
@@ -422,67 +423,58 @@ TSP_provider_t* TSP_consumer_connect_url(const char*  url)
     
   protocol = url_tok;
   p = strstr(url_tok, "://");
-  if(!p)
-    {
-      /* set p to hostname field, if any */
-      p = strstr(url_tok, "//");
-      if(p) p += 2;
-      else p = url_tok; /* may start of string be the hostname ?! */
-
-      /* no protocol specified, use default */
-      protocol = strdup(TSP_PROTOCOL);
-   }
-  else
-    {
-      /* protocol should be OK (start of URL), set p to hostname field */
-      if(p == url_tok) protocol = strdup(TSP_PROTOCOL);
-      *p = '\0';
-      p += 3;
-    }
+  if(!p) {
+    /* set p to hostname field, if any */
+    p = strstr(url_tok, "//");
+    if(p) p += 2;
+    else p = url_tok; /* may start of string be the hostname ?! */
+    
+    /* no protocol specified, use default */
+    protocol = strdup(TSP_DEFAULT_PROTOCOL);
+  } else {
+    /* protocol should be OK (start of URL), set p to hostname field */
+    if(p == url_tok) protocol = strdup(TSP_DEFAULT_PROTOCOL);
+    *p = '\0';
+    p += 3;
+  }
 
   hostname = p;
   p = strstr(hostname, "/");
-  if(p == hostname)
-    {
-      /* no hostname provided, use default & set p to server name field */
-      hostname = strdup("localhost");
-      p += 1;
-    }
-  else if(!p)
-    {
-      /* end of string ... hostname should be OK, set p to the end */
-      p = hostname + strlen(hostname);
-    }
-  else
-    {
-      /* hostname is OK, set p to server name field */
-      *p = '\0';
-      p += 1;
-    }
+  if(p == hostname) {
+    /* no hostname provided, use default & set p to server name field */
+    hostname = strdup("localhost");
+    p += 1;
+  } else if(!p) {
+    /* end of string ... hostname should be OK, set p to the end */
+    p = hostname + strlen(hostname);
+  } else {
+    /* hostname is OK, set p to server name field */
+    *p = '\0';
+    p += 1;
+  }
 
   servername = p;
   p = strstr(servername, ":");
-  if(!p)
-    {
-      /* servername should be OK (or 0 length), set p to number field */
-      p = servername + strlen(servername);
-    }
-  else
-    {
-      /* servername is OK, set p to number field */
-      *p = '\0';
-      p += 1;
-    }
+  if(!p) {
+    /* servername should be OK (or 0 length), set p to number field */
+    p = servername + strlen(servername);
+  } else {
+    /* servername is OK, set p to number field */
+    *p = '\0';
+    p += 1;
+  }
 
-  if(*p)
-    {
-      servernumber = atoi(p);
-      if(errno == EINVAL)
-	servernumber = -1;
+  if(*p) {
+    /* strtol should set errno which is not the case of atoi */
+    endptr = NULL;
+    servernumber = strtol(p,&endptr,10);
+    /* in case we did not convert anything at all */
+    if (endptr == p) {
+      servernumber = -1;
     }
+  }
 
-
-  /** Full URL, or without server name : try to connect to server number **/
+  /* Full URL, or without server name : try to connect to server number */
   if( servernumber >= 0 )
     {
       sprintf(url_lkup, TSP_URL_FORMAT, protocol, hostname, servername, servernumber);
@@ -504,26 +496,23 @@ TSP_provider_t* TSP_consumer_connect_url(const char*  url)
       STRACE_INFO(("No TSP provider on URL <%s>", url_lkup));
       return NULL;
     }
-else
-    {
-      /** Partial URL, without server number : try to find one, by recursion **/
-      int server_max_number = TSP_get_server_max_number();
-       
-      for(i = 0; i < server_max_number; i++)
-	{
-	  sprintf(url_lkup, TSP_URL_FORMAT, protocol, hostname, servername, i);
-	  provider = TSP_consumer_connect_url(url_lkup);
-	  if(provider)
-	    return provider;
-	}
-      STRACE_INFO(("No TSP provider based on URL <%s>", url));
-      return NULL;
-      
+  else {
+    /** Partial URL, without server number : try to find one, by recursion **/
+    int server_max_number = TSP_get_server_max_number();
+    
+    for(i = 0; i < server_max_number; i++) {
+      sprintf(url_lkup, TSP_URL_FORMAT, protocol, hostname, servername, i);
+      provider = TSP_consumer_connect_url(url_lkup);
+      if(provider)
+	return provider;
     }
-
+    STRACE_INFO(("No TSP provider based on URL <%s>", url));
+    return NULL;    
+  }
+  
   STRACE_ERROR(("Cannot parse such URL %s", url));
   return NULL;
-}
+} /* end of TSP_consumer_connect_url */
 
 void TSP_consumer_disconnect_one(TSP_provider_t provider)
 {	
@@ -570,7 +559,7 @@ void TSP_consumer_connect_all(const char*  host_name, TSP_provider_t** providers
 	  STRACE_DEBUG(("Trying to open server No %d", i));
 
 	  /* Is server number 'i' alive ?*/ 
-	  if(TSP_remote_open_server(  TSP_PROTOCOL,
+	  if(TSP_remote_open_server(  TSP_DEFAULT_PROTOCOL,
 				      host_name,
 				      "",
 				      i, 
@@ -581,7 +570,7 @@ void TSP_consumer_connect_all(const char*  host_name, TSP_provider_t** providers
 	      (*providers)[*nb_providers] = TSP_new_object_tsp(server, server_info);
 	      if( 0 == (*providers)[*nb_providers])
 		{
-		  STRACE_ERROR(("TSP_new_object_tsp failedfor No=%d", i));
+		  STRACE_ERROR(("TSP_new_object_tsp failed for No=%d", i));
 		  (*nb_providers) = 0;
 		  return;
 
@@ -1312,17 +1301,17 @@ int TSP_consumer_read_sample(TSP_provider_t provider, TSP_sample_t* sample, int*
   return ret;
 }
 
-TSP_groups_t TSP_test_get_groups(TSP_provider_t provider)
-{
+TSP_groups_t TSP_test_get_groups(TSP_provider_t provider) {
   TSP_otsp_t* otsp = (TSP_otsp_t*)provider;
   
   return otsp->groups;
 }
 
 
-int TSP_consumer_request_async_sample_write(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_write)
+int 
+TSP_consumer_request_async_sample_write(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_write)
 {
- 
+  int *result;
   TSP_async_sample_t async_write;
   int ret = 0;
   TSP_otsp_t* otsp = (TSP_otsp_t*)provider;
@@ -1330,47 +1319,64 @@ int TSP_consumer_request_async_sample_write(TSP_provider_t provider,TSP_consumer
   /* As there are a two level structure for hidding all RPC stuff, we need to copy the struct fields */
   
   async_write.provider_global_index = async_sample_write->provider_global_index;
-  async_write.data.data_val = async_sample_write->value_ptr;
-  async_write.data.data_len = async_sample_write->value_size;
+  async_write.data.data_val         = async_sample_write->value_ptr;
+  async_write.data.data_len         = async_sample_write->value_size;
   
   /* verification of the structure*/
   	
-  if(0 != otsp) {    
-    ret = *(TSP_request_async_sample_write(&async_write,otsp->server));      
-  }
-  else {
-      STRACE_ERROR(("This provider is not instanciate"));
+  if(0 != otsp) { 
+    result = TSP_request_async_sample_write(&async_write,otsp->server);
+    if (result) {
+      ret = *result; 
+    } else {
+      STRACE_DEBUG(("result is <0x%X>\n", result));      
+    }
+  } else {
+    STRACE_ERROR(("This provider is not instanciate"));
   }
      
   STRACE_IO(("-->OUT"));
 	
   return ret;
 	
-}
+} /* TSP_consumer_request_async_sample_write */
 
-int TSP_consumer_request_async_sample_read(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_read)
+int 
+TSP_consumer_request_async_sample_read(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_read)
 {
  
   TSP_async_sample_t* async_read_result;
   TSP_async_sample_t async_read_param;
-
   
-  int ret = 0;
+  int ret = TSP_STATUS_ERROR_UNKNOWN;
   TSP_otsp_t* otsp = (TSP_otsp_t*)provider;
  
   /* update internal RPC structure */
   async_read_param.provider_global_index  = async_sample_read->provider_global_index;
   async_read_param.data.data_val = async_sample_read->value_ptr;
   async_read_param.data.data_len = async_sample_read->value_size;
+
   
   if(0 != otsp) {   
     STRACE_DEBUG(("TSP consumer async read for pgi <%d>\n",async_sample_read->provider_global_index));
+
     async_read_result = TSP_request_async_sample_read(&async_read_param,otsp->server);
-    if (-1 == async_read_result->provider_global_index) {
-      ret = 1;
+    STRACE_DEBUG(("async_read_result is <0x%X>\n", async_read_result));
+
+    /* Provider has probably died */
+    if (async_read_result == NULL) {
+      ret=TSP_STATUS_ERROR_UNKNOWN;
     } else {
-      /* should update value */
-      memcpy(async_sample_read->value_ptr,async_read_result->data.data_val,async_sample_read->value_size);
+      STRACE_DEBUG(("async_read_result->pgi=%d\n", async_read_result->provider_global_index));
+      STRACE_DEBUG(("async_read_result->value_size=%d\n", async_sample_read->value_size));
+      STRACE_DEBUG(("async_read_result->data.data_val = 0x%X\n", async_read_result->data.data_val));
+      if (-1 == async_read_result->provider_global_index) {
+	ret = TSP_STATUS_ERROR_PGI_UNKNOWN;
+      } else {
+	/* should update value */
+	memcpy(async_sample_read->value_ptr,async_read_result->data.data_val,async_sample_read->value_size);
+	ret = TSP_STATUS_OK;
+      }
     }
   }
   else{
@@ -1381,7 +1387,7 @@ int TSP_consumer_request_async_sample_read(TSP_provider_t provider,TSP_consumer_
 	
   return ret;
 	
-}
+} /* TSP_consumer_request_async_sample_read */
 
 void 
 TSP_consumer_print_invalid_symbols(FILE* stream, 
