@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Id: tsp_consumer.h,v 1.19 2004/11/09 22:33:01 erk Exp $
+$Id: tsp_consumer.h,v 1.24 2005/11/01 10:37:12 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -38,6 +38,11 @@ Purpose   : Main interface for the TSP consumer library
 #ifndef _TSP_CONSUMER_H
 #define _TSP_CONSUMER_H
 
+#include <stdio.h>
+
+#include <tsp_abs_types.h>
+#include <tsp_common.h>
+
 /**
  * @defgroup Consumer
  * @ingroup Core
@@ -138,6 +143,14 @@ struct TSP_consumer_information_t
 
 typedef struct TSP_consumer_information_t TSP_consumer_information_t;
 
+typedef struct TSP_consumer_async_sample_t
+{
+    int provider_global_index;
+    void * value_ptr;
+    int    value_size;
+} TSP_consumer_async_sample_t;
+
+
 
 /** Opaque handle type for provider */
 typedef  void* TSP_provider_t;
@@ -152,17 +165,26 @@ typedef  void* TSP_provider_t;
 
 /*--------------------------------- FUNCTIONS -------------------------------*/
 
+/** @} */
+
+BEGIN_C_DECLS
+
 /**
-* Initialisation for TSP library.
-* Call this function before using main(argc, argv) function arguments. This function
-* removes the arguments it knows from the argument list, leaving anything
-* it does not recognize for your application to parse or ignore. 
-* This creates a set of standard arguments accepted by all TSP applications.
-* This function must be called once.
-* @param argc Use the argc main arg before using it
-* @param argv Use the argv main arg before using it
-* @return TRUE = OK
-*/
+ * @addtogroup Consumer
+ * @{
+ */
+
+/**
+ * Initialisation for TSP library.
+ * Call this function before using main(argc, argv) function arguments. This function
+ * removes the arguments it knows from the argument list, leaving anything
+ * it does not recognize for your application to parse or ignore. 
+ * This creates a set of standard arguments accepted by all TSP applications.
+ * This function must be called once.
+ * @param argc Use the argc main arg before using it
+ * @param argv Use the argv main arg before using it
+ * @return TRUE = OK
+ */
 int TSP_consumer_init(int* argc, char** argv[]);
 
 /**
@@ -283,6 +305,18 @@ int TSP_consumer_request_close(TSP_provider_t provider);
  */				  
 int TSP_consumer_request_information(TSP_provider_t provider);
 
+/** 
+ * Request provider information.
+ * Ask the provider informations about several parameters, including
+ * the available symbol list that can be asked.
+ * This function should be called multiple times only to refresh
+ * the structure returned by the TSP_consumer_get_information function.
+ * @param provider The provider handle
+ * @param filter_kind the kind of filter
+ * @param filter_string the filter string
+ * @return TRUE or FALSE. TRUE = OK.
+ */				  
+int TSP_consumer_request_filtered_information(TSP_provider_t provider, int filter_kind, char* filter_string);
 
 /** 
  * Retrieve the provider list.
@@ -356,6 +390,15 @@ int TSP_consumer_read_sample(TSP_provider_t provider,
 			     TSP_sample_t* sample,
 			     int* new_sample);
 
+
+int TSP_consumer_request_async_sample_write(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_write);
+
+int TSP_consumer_request_async_sample_read(TSP_provider_t provider,TSP_consumer_async_sample_t* async_sample_read);
+
+void TSP_consumer_print_invalid_symbols(FILE*, TSP_consumer_symbol_requested_list_t* symbols, const char* tsp_provider_url);
+
 /** @} end group Consumer */ 
+
+END_C_DECLS
     
 #endif /* _TSP_CONSUMER_H */
