@@ -1,6 +1,6 @@
 /*!  \file 
 
-$Header: /cvsroot/tsp/tsp/src/providers/bb_provider/bb_tsp_provider.h,v 1.7 2005/10/09 23:01:25 erk Exp $
+$Header: /cvsroot/tsp/tsp/src/providers/bb_provider/bb_tsp_provider.h,v 1.10 2006/01/22 09:35:15 erk Exp $
 
 -----------------------------------------------------------------------
 
@@ -41,49 +41,47 @@ Purpose   : Blackboard TSP Provider
 
 #include <pthread.h>
 
-#ifdef BB_TSP_PROVIDER_C
-pthread_t glu_thread_id=0;
-#else
-extern pthread_t glu_thread_id;
-#endif
-
 /**
- * Indique la frequence de base du provider TSP
- * @param d_frequence_provider la frequence de base en Hz
- * @return toujours 0, cette fonction n'choue jamais.
- * @ingroup BlackBoard
- */
-int 
-GLU_set_base_frequency(double d_frequence_provider);
-
-/**
- * Initialisation du fournisseur de donne TSP pour un BlackBoard TSP.
+ * Initialize Blackboard TSP provider.
  * On doit passer  cette fonction d'initialisation les 
  * argument en provenance d'un main car la lib TSP peut
  * de cette manire grer des paramtres spcifiques  TSP.
  * Les paramtres qui ne concernent pas TSP seront inchangs.
  * 
- * @param argc Le parametre argc d'un main avant que le main
- *             ne l'utilise.
- * @param argv Le parametre argc d'un main avant que le main
- *             ne l'utilise.
- * @param TSPRunMode mode de dmarrage 0 = non bloquant, 1 = bloquant.
- * @param bbname le nom du blackboard auquel s'attacher
- * @return E_OK si initialisation ralise E_NOK sinon.
+ * @param argc argc coming from main (before the main use it)
+ * @param argv argv coming from main (before the main use it)
+ * @param TSPRunMode running mode, 0 = non blocking, 1 = blocking (never return).
+ * @param bbname the blackboard name we should attach to.
+ * @param frequency the frequency advertised by the bb_provider 
+ * @param acknowledgeCopy 0 if no acknowledgment is to be sent
+ *                        by the provider to the simulation driving
+ *                        sample, otherwise acknowledgment is sent.
+ * @return E_OK if initialization is successfull  E_NOK otherwise.
  * @ingroup BlackBoard
  */
 int32_t
-bb_tsp_provider_initialise(int* argc, char** argv[],int TSPRunMode, const char* bbname);
+bb_tsp_provider_initialise(int* argc, char** argv[],int TSPRunMode, 
+			   const char* bbname,
+			   double frequency, 
+			   int32_t acknowledgeCopy);
 
 /**
- *
+ * Terminate bb_provider nicely.
  */
 int32_t 
 bb_tsp_provider_finalize();
 
+/**
+ * Authorize asynchronous write for BB symbols
+ * specified by its PGI.
+ */
 int32_t 
 bb_tsp_provider_allow_write_symbol(int provider_global_index);
 
+/**
+ * Forbid asynchronous write for BB symbols
+ * specified by its PGI.
+ */
 int32_t 
 bb_tsp_provider_forbid_write_symbol(int provider_global_index);
 
